#!/usr/bin/awk -f
#
# Sentinel - IRC Statistical and Operator Services
# genhtml.awk - HTML dump generator
#
# Copyright W. Campbell and others.  See README for more details
# Some code Copyright: Jonathan George, Kai Seidler, ircd-hybrid Team,
#                      IRCnet IRCD developers.
#
# $Id: genhtml.awk,v 1.80 2003/12/10 02:44:52 wcampbel Exp $

BEGIN {
  print "/* html.c - generated source file to perform HTML dumps of"
  print "**          the Stats data."
  print "** DO NOT EDIT!!!"
  print "*/"
  print " "
  print "#include \"stats.h\""
  print "#include \"struct.h\""
  print "#include \"proto.h\""
  print "#include \"lang.h\""
  print "#include \"slog.h\""
  print "#include \"dlink.h\""
  print " "
  print "void DumpHTML()"
  print "{"
  print "  FILE *fp = fopen(settings.html, \"w\");"
  print "  time_t now = time(NULL);"
  print "  struct Server *s;"
  print "  struct User *u;"
  print "  int i;"
  print "  struct TLD *t;"
  print "  dlink_node *dl;"
  print " "
  print "  int showadmin = 0;"
  print "  int hideuplink = 0;"
  print "  int hideservers = 1;"
  print "  int showinterval = 0;"
  print "#ifdef TRACK_VERSION"
  print "  int hideversion = 0;"
  print "#endif"
  print " "
  print "  if (!fp)"
  print "  {"
  print "    slog(DEFAULT_LOG, L_ERR, \"Unable to open HTML Dump file %s for writing.\","
  print "         settings.html);"
  print "    return;"
  print "  }"
  print " "
  print "  slog(DEFAULT_LOG, L_NOTICE, \"HTML Dump file %s opened\", settings.html);"
  print " "
  print " /* write out the HTML heading lines */"
  print " "
}

{
  if ($1 == "!NETWORK_DATA!") {
    print " "
    print "#ifdef STATSERV"
    print "  s = findserver(settings.name);"
    print "  if (IsSHide(s) && hideservers)"
    print "    fprintf(fp, \"%s %s<BR>\\n\", H_MYNAME, StatServ.nick);"
    print "  else"
    print "    fprintf(fp, \"%s %s!%s@%s on %s<BR>\\n\", H_MYNAME, StatServ.nick, StatServ.user, StatServ.host, settings.name);"
    print "#endif"
    print "    if (me.uplink && !(IsSHide(me.uplink) && hideservers))"
    print "      fprintf(fp, \"%s %s<BR>\\n\", H_MYUPLINK, me.uplink->name);"
    print "  fprintf(fp, \"%s %s<BR>\\n\", H_MYADMIN, HTMLReplace(settings.admin_name));"
    print "  fprintf(fp, \"%s %ld<BR>\\n\", H_CUSER, netinfo.users);"
    print "  fprintf(fp, \"%s %ld %s %s<BR>\\n\", H_MUSER, netinfo.maxusers, H_TIMEPREP, sftime(netinfo.t_maxusers));"
    print "  /* hybrid doesn't propegate AWAY, so don't print it out */"
    print "  fprintf(fp, \"%s %d (%2.0f%%)<BR>\\n\", H_CAWAY, netinfo.away, ( (float) netinfo.away / (float) netinfo.users * 100) );"
    print "  fprintf(fp, \"%s %d %s %s<BR>\\n\", H_MAWAY, netinfo.maxaway, H_TIMEPREP, sftime(netinfo.t_maxaway));"
    print "  fprintf(fp, \"%s %d (%2.0f%%)<BR>\\n\", H_CINVIS, netinfo.invis, ( (float) netinfo.invis / (float) netinfo.users * 100) );"
    print "  fprintf(fp, \"%s %d (%2.0f%%)<BR>\\n\", H_COPER, netinfo.opers, ( (float) netinfo.opers / (float) netinfo.users * 100) );"
    print "  fprintf(fp, \"%s %d %s %s<BR>\\n\", H_MOPER, netinfo.maxopers, H_TIMEPREP, sftime(netinfo.t_maxopers));"
    print "  fprintf(fp, \"%s %d<BR>\\n\", H_CSERVER, netinfo.servers);"
    print "  fprintf(fp, \"%s %d %s %s<BR>\\n\", H_MSERVER, netinfo.maxservers, H_TIMEPREP, sftime(netinfo.t_maxservers));"
    print "  fprintf(fp, \"%s %d<BR>\\n\", H_CCHANNEL, netinfo.channels);"
    print "  fprintf(fp, \"%s %d %s %s<BR>\\n\", H_MCHANNEL, netinfo.maxchannels, H_TIMEPREP, sftime(netinfo.t_maxchannels));"
    print "  fprintf(fp, \"%s %ld %s %ld %s<BR>\\n\", H_NETCLICON, netinfo.connections, H_SINCEST, daily.connections, H_TODAY);"
    print "  fprintf(fp, \"%s %s<BR>\\n\", H_LASTUP, sftime(now));"
    print "  fprintf(fp, \"%s %s<BR>\\n\", H_CURRUP, timediff(now - me.t_start));"
    print "  if (showinterval)"
    print "    fprintf(fp, \"%s %s<BR>\\n\", H_INTERVAL, timediff(settings.html_time));"
    print " "
  } else if ($1 == "!STATS_VERSION!") {
    print " "
    print "  fprintf(fp, \"%s %s %s<BR>\\n\", H_STATSVER, stats_version, VersionFlags());"
    print " "
  } else if ($1 == "!SERVER_LIST!") {
    print " "
    print "  for (i=0; i < S_TABLE_SIZE; i++)"
    print "    DLINK_FOREACH(dl, serverhash[i].head)"
    print "    {"
    print "      s = dl->data;"
    print "      if (!(IsSHide(s) && hideservers))"
    print "        fprintf(fp, \"<A HREF=\\\"#%s\\\">%s</A><BR>\\n\", s->name, s->name);"
    print "    }"
    print " "
  } else if ($1 == "!TABLE_LIST!") {
    print " "
    print "  fprintf(fp, \"<TABLE BORDER=\\\"1\\\"><TR><TH>%s</TH><TH>%s</TH><TH>%s</TH><TH>%s</TH></TR>\", H_SERVER, H_USERMAX, H_OPERMAX, H_PERCENT);"
    print "  for (i=0; i < S_TABLE_SIZE; i++)"
    print "  {"
    print "    DLINK_FOREACH(dl, serverhash[i].head)"
    print "    {"
    print "      s = dl->data;"
    print "      if (!(IsSHide(s) && hideservers))"
    print "      {"
    print "          fprintf(fp, \"<TR><TD><A HREF=\\\"#%s\\\">%s</A></TD><TD>%d/%d</TD><TD>%d/%d</TD><TD>%2.0f%%</TD></TR>\\n\","
    print "                  s->name, s->name, s->users, s->stats->maxusers, s->opers, s->stats->maxopers, (float) s->users / (float) netinfo.users * 100);"
    print "      }"
    print "    }"
    print "  }"
    print "  fprintf(fp, \"</TABLE>\\r\");"
    print " "
  } else if ($1 == "!MAP_LIST!") {
    print " "
    print "#ifdef SPLITSERV"
    print "  html_map(fp, map_head, 0, NULL);"
    print "#endif"
    print " "
  } else if ($1 == "!TOP_CHANNELS!") {
    print " "
    print "  PrintTopChannels(fp, settings.topchannum);"
    print " "
  } else if ($1 == "!BOT_LIST!") {
    print " "
    print "#ifdef STATSERV"
    print "  fprintf(fp, \"%s!%s@%s<BR>\\n\", StatServ.nick, StatServ.user, StatServ.host);"
    print "#endif"
    print "#ifdef SPLITSERV"
    print "  fprintf(fp, \"%s!%s@%s<BR>\\n\", SplitServ.nick, SplitServ.user, SplitServ.host);"
    print "#endif"
    print "  if (showadmin)"
    print "    fprintf(fp, \"%s!%s@%s<BR>\\n\", AdminServ.nick, AdminServ.user, AdminServ.host);"
    print "#ifdef JUPESERV"
    print "    fprintf(fp, \"%s!%s@%s<BR>\\n\", JupeServ.nick, JupeServ.user, JupeServ.host);"
    print "#endif"
    print "#ifdef LISTSERV"
    print "    fprintf(fp, \"%s!%s@%s<BR>\\n\", ListServ.nick, ListServ.user, ListServ.host);"
    print "#endif"
    print "#ifdef MONSERV"
    print "    fprintf(fp, \"%s!%s@%s<BR>\\n\", MonServ.nick, MonServ.user, MonServ.host);"
    print "#endif"
    print " "
  } else if ($1 == "@SHOW_ADMIN@") {
    print " "
    print "  showadmin = 1;"
    print " "
  } else if ($1 == "@HIDE_UPLINK@") {
    print " "
    print "  hideuplink = 1;"
    print " "
  } else if ($1 == "@SHOW_INTERVAL@") {
    print " "
    print "  showinterval = 1;"
    print " "
  } else if ($1 == "@HIDE_VERSION@") {
    print " "
    print "#ifdef TRACK_VERSION"
    print "  hideversion = 1;"
    print "#endif"
    print " "
  } else if ($1 == "@SHOW_HIDDEN@") {
    print " "
    print "  hideservers = 0;"
    print " "
  } else if ($1 == "!TLD_TABLE!") {
    print " "
    print "  for (i = 0; i < T_TABLE_SIZE; i++)"
    print "  {"
    print "    for(t = tldlist[i]; t; t = t->next)"
    print "    {"
    print "      if (t->users > 0)"
    print "      {"
    print "        fprintf(fp, \"%d %s %s %s (%s) (%2.0f%%)<BR>\\n\", t->users,"
    print "        (t->users == 1) ? H_W_USER : H_W_USERS, H_TLDPREP, t->country, t->tld,"
    print "        (float)t->users / (float)netinfo.users * 100);"
    print "      }"
    print "    }"
    print "  }"
    print " "
  } else if ($1 == "!OPER_LIST!") {
    print " "
    print "  fprintf(fp, \"<OL>\\n\");"
    print "  DLINK_FOREACH(dl, global_oper_list.head)"
    print "  {"
    print "    u = (struct User *) dl->data;"
    print "    if (!(IsSHide(u->server) && hideservers) && IsOper(u))"
    print "    {"
    print "#ifdef OPER_MODES"
    print "      fprintf(fp, \"<LI>[%c] %s %s %s<BR>\\n\", opermode(u), u->nick, H_SERVPREP, u->server->name);"
    print "#else"
    print "      fprintf(fp, \"<LI>%s %s %s<BR>\\n\", u->nick, H_SERVPREP, u->server->name);"
    print "#endif"
    print "    }"
    print "  }"
    print "  fprintf(fp, \"</OL>\\n\");"
    print " "
  } else if ($1 == "!SPLIT_LIST!") {
    print " "
    print "#ifdef SPLITSERV"
    print "  {"
    print "    int i;"
    print "    int count = 0;"
    print "    char * td;"
    print "    struct SpServer *spl;"
    print " "
    print "    DLINK_FOREACH(dl, split_table.head)"
    print "    {"
    print "      spl = (struct SpServer *) dl->data;"
    print "      td = timediff(now - spl->ts);"
    print "      count++;"
    print "      fprintf(fp, \"%s %s %s<BR>\\n\", spl->name, H_SPLITPREP, td);"
    print "    }"
    print "    if (!count)"
    print "    {"
    print "      fprintf(fp, \"%s<BR>\\n\", H_NOSPLITS);"
    print "    }"
    print "  }"
    print "#endif"
    print " "
  } else if ($1 == "!VERSION_MAP!") {
    print " "
    print "#ifdef TRACK_VERSION"
    print "  fprintf(fp, \"<TABLE><TR><TH>%s</TH><TH>%s</TH></TR>\\r\", H_SERVER, H_VERSION);"
    print "  for (i=0; i < S_TABLE_SIZE; i++)"
    print "  {"
    print "    DLINK_FOREACH(dl, serverhash[i].head)"
    print "    {"
    print "      s = dl->data;"
    print "      if (!(IsSHide(s) && hideservers))"
    print "      {"
    print "          fprintf(fp, \"<TR><TD>%s</TD><TD>%s</TD></TR>\\r\", s->name, (s->version && s->version[0]) ?  HTMLReplace(s->version) : H_UNKVERSION);"
    print "      }"
    print "    }"
    print "  }"
    print "  fprintf(fp, \"</TABLE>\\r\");"
    print "#endif"
    print " "
  } else if ($1 == "!SERVER_DATA!") {
    print " "
    print "  for (i=0; i < S_TABLE_SIZE; i++)"
    print "    DLINK_FOREACH(dl, serverhash[i].head)"
    print "    {"
    print "      s = dl->data;"
    print "      if (!(IsSHide(s) && hideservers))"
    print "      {"
    print "        fprintf(fp, \"%s\\n\", H_SEP);"
    print "        fprintf(fp, \"<A NAME=\\\"%s\\\"></A>\", s->name);"
    print "        fprintf(fp, \"%s%s%s\\n\", H_SERVER_BEG, s->name, H_SERVER_END);"
    print "        fprintf(fp, \"%s %s<BR>\\n\", H_DESC, HTMLReplace(s->desc));"
    print "#ifdef TRACK_VERSION"
    print "        if (!hideversion && s->version && s->version[0])"
    print "          fprintf(fp, \"%s %s<BR>\\n\", H_SD_VERSION, HTMLReplace(s->version));"
    print "#endif"
    print "#ifdef TRACK_ADMIN"
    print "        if (s->admin1[0] != '\\0')"
    print "           fprintf(fp, \"%s %s<BR>\\n\", H_ADMIN, HTMLReplace(s->admin1));"
    print "        if (s->admin2[0] != '\\0')"
    print "           fprintf(fp, \"%s %s<BR>\\n\", H_ADMIN, HTMLReplace(s->admin2));"
    print "        if (s->admin3[0] != '\\0')"
    print "           fprintf(fp, \"%s %s<BR>\\n\", H_ADMIN, HTMLReplace(s->admin3));"
    print "#endif"
    print "        fprintf(fp, \"%s %d<BR>\\n\", H_CUSER, s->users);"
    print "        fprintf(fp, \"%s %ld %s %s<BR>\\n\", H_MUSER, s->stats->maxusers, H_TIMEPREP, sftime(s->stats->t_maxusers));"
    print "        fprintf(fp, \"%s %d<BR>\\n\", H_COPER, s->opers);"
    print "        fprintf(fp, \"%s %d %s %s<BR>\\n\", H_MOPER, s->stats->maxopers, H_TIMEPREP, sftime(s->stats->t_maxopers));"
    print "        fprintf(fp, \"%s %d<BR>\\n\", H_CAWAY, s->away);"
    print "        fprintf(fp, \"%s %d %s %s<BR>\\n\", H_MAWAY, s->stats->maxaway, H_TIMEPREP, sftime(s->stats->t_maxaway));"
    print "        fprintf(fp, \"%s %s<BR>\\n\", H_CONSINCE, sftime(s->connected_since));"
    print "        if (!hideuplink)"
    print "        {"
    print "          if (s->uplink)"
    print "          {"
    print "            if (!(hideservers && (s->uplink->flags & SFLAG_HIDE)))"
    print "              fprintf(fp, \"%s <A HREF=\\\"#%s\\\">%s</A><BR>\\n\", H_UPLINK, s->uplink->name, s->uplink->name);"
    print "          }"
    print "        }"
    print "    }"
    print "  }"
    print " "
  } else if ($1 == "!DYN_HEADER!") {
    print " "
    print "  fprintf(fp, \"<TITLE>%s %s</TITLE>\\n\", H_TITLE, sftime(now));"
    print "  fprintf(fp, \"<META HTTP-EQUIV=\\\"Refresh\\\" CONTENT=\\\"%d\\\">\\n\", settings.html_time);"
    print " "
  } else if ($1 == "!SERVICE_LIST!") {
    print " "
  } else {
    print "     fprintf(fp, \""$0"\\n\");"
  }
}

END {
  print "  fclose(fp);"
  print "}"
}
